"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.EAxisSideClipping = void 0;
/**
 * Defines the Clipping rule for {@link https://www.scichart.com/javascript-chart-features | Javascript 3D Chart} Axis
 * @remarks
 * Set Clipping rule on the {@link AxisBase3D.negativeSideClipping} and {@link AxisBase3D.positiveSideClipping}
 */
var EAxisSideClipping;
(function (EAxisSideClipping) {
    /**
     * Default {@link https://www.scichart.com/javascript-chart-features | Javascript 3D Chart} Axis clipping is
     * {@link AxisCore.visibleRange} for X and Z Axis, and None for Y Axis
     * @remarks
     * Data from {@link https://www.scichart.com/javascript-chart-features | 3D Javascript Chart} types can spill
     * out beyond the confines of the Axis for the YAxis (up/down)
     * but be clipped to the {@link AxisCore.visibleRange} for the Z/X Axis
     */
    EAxisSideClipping[EAxisSideClipping["Default"] = 0] = "Default";
    /**
     * 3D Axis Clipping is disabled
     * @remarks
     * Data from {@link https://www.scichart.com/javascript-chart-features | 3D Javascript Chart}
     * types can spill out beyond the confines of the Axis.
     */
    EAxisSideClipping[EAxisSideClipping["None"] = 1] = "None";
    /**
     * RenderableSeries are clipped by {@link AxisCore.visibleRange}
     * @remarks
     * If a {@link https://www.scichart.com/javascript-chart-features | Javascript Chart} type such as a
     * 3D Scatter or 3D Surface mesh plot are plotted on this axis,
     * the data will be clipped to be invisible outside the {@link AxisCore.visibleRange}
     */
    EAxisSideClipping[EAxisSideClipping["VisibleRange"] = 2] = "VisibleRange";
})(EAxisSideClipping = exports.EAxisSideClipping || (exports.EAxisSideClipping = {}));
