import { CanvasTexture } from "../../../../Charting/Visuals/TextureManager/CanvasTexture";
import { Size } from "../../../../types/Size";
import { TSciChart3D } from "../../../../types/TSciChart3D";
import { MeshColorPalette } from "./MeshColorPalette";
/**
 * Optional parameters passed to the constructor of a {@link SolidColorBrushPalette}
 */
export interface ISolidColorBrushPaletteOptions {
    /**
     * Gets or sets the solid color fill as an HTML Color code
     */
    fill?: string;
}
/**
 * Provides a solid color (single color) palette which may be applied to the {@link SurfaceMeshRenderableSeries3D.meshColorPalette}
 * property
 */
export declare class SolidColorBrushPalette extends MeshColorPalette {
    private fillProperty;
    /**
     * Creates an instance of the {@link SolidColorBrushPalette}
     * @param webAssemblyContext The {@link TSciChart3D | SciChart 3D WebAssembly Context}
     * containing native methods and access to our WebGL2 Engine and WebAssembly numerical methods
     * @param options optional parameters of type {@link ISolidColorBrushPaletteOptions} passed to the constructor
     */
    constructor(webAssemblyContext: TSciChart3D, options?: ISolidColorBrushPaletteOptions);
    /**
     * Gets or sets the solid color fill as an HTML Color code
     */
    get fill(): string;
    /**
     * Gets or sets the solid color fill as an HTML Color code
     */
    set fill(fill: string);
    /**
     * @inheritDoc
     */
    getTexture(size: Size): CanvasTexture;
}
